/*
 * scanner.c
 *
 * Created: 2014-04-04 14:37:41
 *  Author: dsosnowski
 */ 

#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include "scanner.h"
//#include "ee_mem.h"
#include "defs.h"

//CLK = 8MHz
//timer f = 8000000/256/64 = 122 Hz
//timer t = 1/244 = 4,096 ms
//1tick = 16us, 889/8 = 55 ticks
//			  1778/8 = 111 ticks


struct {
	volatile char status;	
	volatile char result;
	volatile char index;
	char buffer[SCAN_BUFF_SIZE];
} scan;


//------------------------
void ScannerInit (void)
{
	IR_PORT |= (1<<IR_PIN);
	INT0_CONFIG;
	TCCR0A = 0;
	sei();
}

//------------------------
char ScannerStatus(void)
{
	return scan.status;
}

//------------------------
void ScannerStart(void)
{
	char i;
	i = 0;
	while (i < SCAN_BUFF_SIZE)
	{
		scan.buffer[i] = 0;
		i++;
	}
	
	scan.result = 0;
	scan.index = 0;
	scan.status = SCAN_FIND;
	
	PRESCALER_RESET;
	TIMER_RESET;
	TIMER_CLEAR;
	TIMER_ENABLE;
	TIMER_START;
	
	INT0_CLEAR;
	INT0_ENABLE;		
}

//------------------------
void ScannerStop(void)
{
	INT0_DISABLE;
	TIMER_STOP;
	TIMER_DISABLE;
	scan.status = SCAN_IDLE;
}

//------------------------
SIGNAL(INT0_vect)
{
	if (scan.status == SCAN_FIND)
	{
		TIMER_RESET;
		PRESCALER_RESET;
	}
	else if (scan.status == SCAN_START)
	{
		if ((IR1_GET) == 0)				//IR pulse
		{
			scan.index = 0;
			scan.status = SCAN_PROCESS;
			TIMER_CLEAR;
			TIMER_START;
		}
	}
	else if (scan.status == SCAN_PROCESS)
	{
		TIMER_STOP;
		scan.buffer[scan.index] = TIMER_VALUE;
		PRESCALER_RESET;
		TIMER_RESET;
		TIMER_START;

		scan.index++;
		if (scan.index < SCAN_BUFF_SIZE) 
			scan.buffer[scan.index] = 0;
		else
		{
			INT0_DISABLE;
			TIMER_STOP;
			TIMER_DISABLE;
			TIMER_CLEAR;
			scan.result = scan.index;
			scan.status = SCAN_COMPLETE; 
		}
	}
}

//-----------------------------------
SIGNAL(TIMER0_OVF_vect)
{
	TIMER_STOP;
	if (scan.index > SCAN_BUFF_MIN)
	{
		INT0_DISABLE;
		TIMER_DISABLE;
		scan.result = scan.index;
		scan.status = SCAN_COMPLETE;		
	}
	else
	{
		scan.result = 0;
		scan.index = 0;
		scan.status = SCAN_START;
			
		TIMER_RESET;
		PRESCALER_RESET;
	}
}

//-----------------------------------
void ScannerSave(char cell)
{
	char i;
	unsigned int address;
	
	if ((cell) && (cell <= SCAN_MAX_CELL))
	{
		ScannerStop();
		address = ((cell - 1) * SCAN_BUFF_SIZE);
		i = 0;		
		while (i <  SCAN_BUFF_SIZE)
		{
			EEwrite(address, scan.buffer[i]);
			address++;
			i++;
		}	
	}
}

//-----------------------------------
void ScannerLoad(char cell)
{
	char i;
	char temp;
	unsigned int address;
	
	if ((cell) && (cell <= SCAN_MAX_CELL))
	{
		address = ((cell - 1) * SCAN_BUFF_SIZE);
		i = 0;
		while (i < SCAN_BUFF_SIZE)
		{
			temp = EEread(address);
			if (!((temp) && (temp < 0xFF)))
				i = SCAN_BUFF_SIZE;
			address++;
			i++;
		}
	}
}

//-----------------------------------
char ScannerComp(void)
{
	char cell;
	unsigned int address;
	char i;
	char temp;
	char min;
	char max;
	char compare;
	char result;
	

	ScannerStop();
	result = 0;
	cell = 0;
	
		while (cell < SCAN_MAX_CELL)
		{
			address = (cell * SCAN_BUFF_SIZE);
			compare = 1;						//set at pass
			i = 0;
			
			while (i < SCAN_BUFF_SIZE)
			{
				temp = EEread(address);
				if ((temp) && (temp < 0xFF))	//if right value
				{
					if (temp < SCAN_PULSE_MIN) min = 1;
					else min = (temp - SCAN_PULSE_TOLERANCE);
					if (temp > SCAN_PULSE_MAX) max = 254;
					else max = (temp + SCAN_PULSE_TOLERANCE);
					if ((scan.buffer[i] < min) || (scan.buffer[i] > max)) 
					{
						compare = 0;			//not equal
						i = SCAN_BUFF_SIZE;		//exit while
					}
				}
				else
				{
					if (i == 0) compare = 0;	//if first cycle skip then empty cell
					//compare = 0;				//not equal
					i = SCAN_BUFF_SIZE;			//exit while
				}
				
				address++;
				i++;
			}
			
			if (compare) 
			{
				result = (cell + 1);
				cell = SCAN_MAX_CELL;
			}
			else cell++;
		}
	return result;
}

//---------------------------------
void EEwrite(unsigned int address, char data)
{
	while(EECR & (1<<EEPE)); 		//Wait for completion of previous write

	EEAR = address;				//Set up address and data registers
	EEDR = data;

	EECR |= (1<<EEMPE);				//Write logical one to EEMPE
	EECR |= (1<<EEPE);				//Start eeprom write by setting EEPE
}

//---------------------------------
char EEread(unsigned int address)
{
	while(EECR & (1<<EEPE));		//Wait for completion of previous write

	EEAR = address;				//Set up address register
	EECR |= (1<<EERE);				//Start eeprom read by writing EERE

	return EEDR;					//Return data from data register
}